/*----------------------------------------------------------------------
 * $Id: bwFilter.cc,v 1.12 2004/04/20 01:07:08 ganzhorn Exp $             
 *                                                                      
 * Copyright (C)   2004            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include "bwFilter.h"

#include <qlayout.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qimage.h>
#include <qlabel.h>
#include <qsizepolicy.h>
#include <qtooltip.h>

// #define LABEL_FORMAT "%.0f%%"
#define LABEL_FORMAT "%.0f"

const int LCL_maxLuminosity = 200;

bwFilter::bwFilter(
   Qt::Orientation o,
   QWidget    *parent,
   const char *name
   )
   : QFrame(parent,name), m_luminosity(100),
     m_updating(false), m_inPreview(false)
{
   m_color[0] = .11;
   m_color[1] = .59;
   m_color[2] = .30;

   QBoxLayout *bl;
   QBoxLayout *topLayout;
   QBoxLayout *rc[3];
   if( Qt::Horizontal == o ) {
      topLayout = new QHBoxLayout(this);
      bl = new QVBoxLayout(topLayout);
      rc[0]  = new QHBoxLayout(bl);
      rc[1]  = new QHBoxLayout(bl);
      rc[2]  = new QHBoxLayout(bl);
//      this->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Minimum);
   } else if( Qt::Vertical ) {
      topLayout = new QVBoxLayout(this);
      bl = new QHBoxLayout(topLayout);
      rc[0]  = new QVBoxLayout(bl);
      rc[1]  = new QVBoxLayout(bl);
      rc[2]  = new QVBoxLayout(bl);
//      this->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
   }
   const int pageStep = 10;
   QLabel    *label[3];
   const int low  = -200;
   const int high = 200;
   const char * labelText[] = {
      "R",
      "G",
      "B"
   };
   const QColor colors[] = {
      Qt::red,
      Qt::green,
      Qt::blue
   };

   for( int i = 0; i < 3; ++i ) {
      m_preview[i] = new QRadioButton(this);
      label[i]     = new QLabel(labelText[i],this);
      m_slider[i]  = new QSlider(low,high,pageStep,(int)(m_color[i]*100),o,
				 this);
      m_lock[i]    = new QCheckBox(this);

      QToolTip::add(m_preview[i],"Channel Preview");
      QToolTip::add(m_lock[i],"Lock Channel");
      QToolTip::add(m_slider[i],"Adjust Channel");

//      m_slider[i]->setTickInterval(50);
//      m_slider[i]->setTickmarks(QSlider::Left);

      m_slider[i]->setSizePolicy( QSizePolicy(QSizePolicy::Fixed,
					      QSizePolicy::Expanding) );

      rc[i]->addWidget(m_preview[i]);
      rc[i]->addWidget(label[i]);
      rc[i]->addWidget(m_slider[i]);
      QString percent;
      percent.sprintf(LABEL_FORMAT,m_color[i]*100);
      label[i] = new QLabel(percent,this);
      label[i]->setPaletteForegroundColor(colors[i]);
      label[i]->setSizePolicy( QSizePolicy(QSizePolicy::Fixed,
      QSizePolicy::Fixed) );
      rc[i]->addWidget(label[i],500);
      rc[i]->addWidget(m_lock[i]);
   }
   connect(m_slider[0],SIGNAL(valueChanged(int)),this,SLOT(setR(int)));
   connect(m_slider[1],SIGNAL(valueChanged(int)),this,SLOT(setG(int)));
   connect(m_slider[2],SIGNAL(valueChanged(int)),this,SLOT(setB(int)));
   
   connect( m_preview[0],SIGNAL(toggled(bool)),this,SLOT(previewR()) );
   connect( m_preview[1],SIGNAL(toggled(bool)),this,SLOT(previewG()) );
   connect( m_preview[2],SIGNAL(toggled(bool)),this,SLOT(previewB()) );

   connect( this,SIGNAL(luminosityR(const QString&)),label[0],
	    SLOT(setText(const QString&)) );
   connect( this,SIGNAL(luminosityG(const QString&)),label[1],
	    SLOT(setText(const QString&)) );
   connect( this,SIGNAL(luminosityB(const QString&)),label[2],
	    SLOT(setText(const QString&)) );
   
   QSlider *luminosity = new QSlider(0,LCL_maxLuminosity,10,
				     100,Qt::Horizontal,this);
   QToolTip::add(luminosity,"Exposure");
   topLayout->addWidget(luminosity);
   connect(luminosity,SIGNAL(valueChanged(int)),this,SLOT(setLuminosity(int)));
   QLabel * luminosityL = new QLabel("100%",this);
   topLayout->addWidget(luminosityL,0,Qt::AlignHCenter);
   connect( this,SIGNAL(luminosity(const QString&)),
	    luminosityL,SLOT(setText(const QString&)) );
}

void
bwFilter::filter(
   QImage &image
   ) const
{
   int checked = -1;
   for( int i = 0; i < 3; ++i ) {
      if( m_preview[i]->isChecked() ) {
	 checked = i;
      }
   }
   if( -1 == checked ) {
      double scale = (double)m_luminosity/100.0;
      convert(image,r()*scale,g()*scale,b()*scale);
   } else {
      for( int x = 0; x < image.width(); ++x ) {
	 for( int y = 0; y < image.height(); ++y ) {
	    QRgb rgb = image.pixel(x,y);
	    int  grey = 0;
	    switch( checked ) {
	       case 0:
		  grey = qRed(rgb);
		  break;
	       case 1:
		  grey = qGreen(rgb);
		  break;
	       case 2:
		  grey = qBlue(rgb);
		  break;
	    }
	    image.setPixel(x,y,qRgb(grey,grey,grey));
	 }
      }
   }
}

void
bwFilter::convert(
   QImage &image,
   double r,
   double g,
   double b
   )
{
   // faster row by col? or col by row?
   int max = (1 << image.depth()/4)-1;
   for( int x = 0; x < image.width(); ++x ) {
      for( int y = 0; y < image.height(); ++y ) {
	 QRgb rgb = image.pixel(x,y);
	 int grey = (int) (qRed(rgb)*r +
			   qGreen(rgb)*g +
			   qBlue(rgb)*b);
	 // clamp the value
	 if( grey < 0 ) {
	    grey = 0;
	 } else if( grey > max ) {
	    grey = max;
	 }
	 image.setPixel(x,y,qRgb(grey,grey,grey));
      }
   }
}

void
bwFilter::setR(
   int r
   )
{
   m_color[0] = (double)r/100;
   update(0);
}

void
bwFilter::setG(
   int g
   )
{
   m_color[1] = (double)g/100;
   update(1);
}

void
bwFilter::setB(
   int b
   )
{
   m_color[2] = (double)b/100;
   update(2);
}

void::
bwFilter::update(
   int idx
   )
{
   if( m_updating ) {
      return;
   }
   int coef[3] = {1,1,1};
   int cnt = 3;
   for( int i = 0; i < 3; ++i ) {
      if( idx == i || m_lock[i]->isChecked()) {
	 coef[i] = 0;
	 --cnt;
      }
   }
   if( cnt ) {
      double delta = -1*(m_color[0]+m_color[1]+m_color[2]-1)/cnt;
      for( int i = 0; i < 3; ++i ) {
	 if( coef[i] ) {
	    m_color[i] += delta;
	 }
      }
   }
   m_updating = true;
   for( int i = 0; i < 3; ++i ) {
      m_slider[i]->setValue((int)(m_color[i]*100));
   }
   m_updating = false;
   QString label;
   label.sprintf(LABEL_FORMAT,m_color[0]*100);
   emit luminosityR(label);
   label.sprintf(LABEL_FORMAT,m_color[1]*100);
   emit luminosityG(label);
   label.sprintf(LABEL_FORMAT,m_color[2]*100);
   emit luminosityB(label);
   emit filterChanged();
}

void
bwFilter::preview(
   int idx
   )
{
   if( !m_inPreview ) {
      m_inPreview = true;
      if( m_preview[idx]->isChecked() ) {
	 for(int i = 0; i < 3; ++i) {
	    if( i != idx ) {
	       m_preview[i]->setChecked(false);
	    }
	 }
      }
      update(idx);
      m_inPreview = false;
   }
}

void
bwFilter::previewR()
{
   preview(0);
}

void
bwFilter::previewG()
{
   preview(1);
}

void
bwFilter::previewB()
{
   preview(2);
}

void
bwFilter::setLuminosity(
   int l
   )
{
   if( l < 0 ) {
      l = 0;
   } else if( l > LCL_maxLuminosity ) {
      l = LCL_maxLuminosity;
   }
   m_luminosity = l;
   QString label;
   label.sprintf(LABEL_FORMAT,(double)m_luminosity);
   emit luminosity(label);
   update(-1);
}
