#ifndef HISTORY_H
#define HISTORY_H
/*----------------------------------------------------------------------
 * $Id: history.h,v 1.2 2004/04/09 02:00:59 ganzhorn Exp $             
 *                                                                      
 * Copyright (C)   2004            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include <deque>

template <class _valueType_t>
class history {
  public:
   history(size_t size=100) {
      m_limit = size;
      m_pos   = m_data.end();
   }
   ~history() {/*noop*/}
   
   const _valueType_t& push( const _valueType_t &value ) {
      if( m_data.size() == m_limit ) {
	 bool first = (m_pos == m_data.begin());
	 m_data.pop_front();
	 if( first ) {
	    m_pos = m_data.begin();
	 } else {
	    ++m_pos;
	 }
      } else if( m_pos != m_data.end() ) {
	 // don't push the head value
	 if( current() == value ) {
	    return current();
	 }
	 ++m_pos;
      } 
      // clear history to the end
      m_data.erase(m_pos,m_data.end());
      m_data.push_back(value);
      m_pos = m_data.end();
      --m_pos;
      return *m_pos;
   }
   const _valueType_t& current() const {
      return *m_pos;
   }
   bool next() {
      if( m_pos == m_data.end() ) {
	 return false;
      }
      container_t::iterator itr = m_pos;
      ++itr;
      if( itr != m_data.end() ) {
	 m_pos = itr;
	 return true;
      } else {
	 return false;
      }
   }
   bool prev() {
      if( m_pos != m_data.begin() ) {
	 --m_pos;
	 return true;
      } else {
	 return false;
      }
   }
   
  protected:
   typedef std::deque<_valueType_t> container_t;
   size_t      m_limit;
   container_t m_data;
   container_t::iterator m_pos;
};

#endif
