/*----------------------------------------------------------------------
 * $Id: imageDisplay.cc,v 1.7 2004/04/09 02:00:59 ganzhorn Exp $             
 *                                                                      
 * Copyright (C)   2004            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include <iostream>

#include <qsize.h>
#include <qpainter.h>
#include <qfiledialog.h>

#include "imageDisplay.h"

ImageDisplay::ImageDisplay( 
    QWidget * parent,
    const char * name )
    : QFrame(parent,name)
{
    setMinimumHeight(200);
}

ImageDisplay::~ImageDisplay()
{}

void
ImageDisplay::paintEvent(
    QPaintEvent * event
    )
{
    QPainter painter(this);
    painter.setClipRect(event->rect());
    painter.drawPixmap(0, 0, m_pixmap);
}    

void 
ImageDisplay::resizeEvent( 
    QResizeEvent * e 
    )
{
    if( isHidden() ) {
	return;
    }
    updatePixmap( e->size() );
}

void
ImageDisplay::updatePixmap(
    const QSize& size
    )
{
   if( m_image.isNull() ) {
      m_resized.reset();
   } else {
      m_resized = m_image.smoothScale( size.width(),size.height(),
				       QImage::ScaleMin) ;
   }
   emit imageChanged(m_resized);
   m_pixmap.convertFromImage( m_resized );
}

void
ImageDisplay::setImage(
    const QString& filename
    )
{
    if( filename != m_filename ) {
	m_filename = filename;
	m_image.load(filename);
	refresh();
    }
}

void
ImageDisplay::refresh()
{
   updatePixmap( size() );
   repaint();
}

void
ImageDisplay::saveAs()
{
   if( !m_image.isNull() ) {
      QString starting = m_filename;
      int slash  = starting.findRev('/');
      int period = starting.findRev('.');
      if( slash < period ) {
	 QString ext = starting.right(starting.length()-period);
	 starting.remove(period,starting.length());
	 starting += "-bw" + ext;
      } else {
	 // no extension?
	 starting.remove(slash+1,starting.length());
      }
      QString filename = QFileDialog::getSaveFileName(starting,
					      "Images (*.jpg *.png *.gif)");
      if( "" != filename ) {
	 QImage saveImage = m_image;
	 emit imageChanged(saveImage);
	 if( !saveImage.save(filename,QImage::imageFormat(m_filename),100) ) {
	    printf("Save failed\n");
	 }
      }
   }
}
