/*----------------------------------------------------------------------
 * $Id: main.cc,v 1.25 2004/04/20 01:07:08 ganzhorn Exp $             
 *                                                                      
 * Copyright (C)   2004            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include <iostream>
#include <exception>

#include <qapplication.h>
#include <qsplitter.h>
#include <qregexp.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qprogressdialog.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qiconset.h>
#include <qtoolbutton.h>

#include "main.h"
#include "imageDisplay.h"
#include "bwFilter.h"

#include "left.xpm"
#include "right.xpm"

using namespace std;

static int LCL_thumbWidth  = 120;
static int LCL_thumbHeight = 80;

MainWindow::MainWindow(
    QWidget    *parent,
    const char *name,
    WFlags      f
    )
   : QMainWindow(parent,name,f), m_imagedisplay(NULL), m_inHistory(false)
{
    // add the URL toolbar
    QToolBar *tb = new QToolBar("URL Tools",this);
    tb->setHorizontallyStretchable(true);
    tb->setCloseMode( QDockWindow::Undocked );
    QPixmap prevPM(left_xpm);
    QPixmap nextPM(right_xpm);
    QIconSet isPrev(prevPM);
    QIconSet isNext(nextPM);
    QToolButton *prev = new QToolButton(tb);
    QToolButton *next = new QToolButton(tb);
    prev->setIconSet(isPrev);
    next->setIconSet(isNext);
    connect( prev,SIGNAL(clicked()),this,SLOT(prev()) );
    connect( next,SIGNAL(clicked()),this,SLOT(next()) );
    QLineEdit *le = new QLineEdit(tb);
    connect( le,SIGNAL(textChanged(const QString&)),
		       this,SLOT(cd(const QString&)));
    le->show();
    connect(this,SIGNAL(dirChanged(const QString &)),
	    le,SLOT(setText(const QString &)));

   // create the Dir Browser toolbar
    tb = new QToolBar("Dir Browser",this);
    tb->setVerticallyStretchable(true);
    tb->setCloseMode( QDockWindow::Undocked );
    moveDockWindow(tb,Left);
    m_listbox = new QListBox(tb);
    m_listbox->setAutoScrollBar(true);
    cwd()->setSorting(QDir::DirsFirst);
    connect( m_listbox,SIGNAL(selected(const QString&)),
	     this,SLOT(cd(const QString&)));
    tb->addSeparator();
    bwFilter * filter = new bwFilter(Qt::Vertical,tb);
    tb->addSeparator();

    // create main pane
    m_splitter = new QSplitter(this);
    m_splitter->setOrientation(Qt::Vertical);

    // add the icon view for thumbnails
    m_iconview = new QIconView(m_splitter);
    m_iconview->setResizeMode( QIconView::Adjust );
    m_iconview->setAutoArrange( true );
    m_iconview->setItemsMovable( false );
    connect( m_iconview,SIGNAL(selectionChanged()),this,SLOT(selectionChanged()));

    // add the image display
    m_imagedisplay = new ImageDisplay(m_splitter);
    m_imagedisplay->hide();

    connect(m_imagedisplay,SIGNAL(imageChanged(QImage&)),
	    filter,SLOT(filter(QImage&)));
    connect(filter,SIGNAL(filterChanged()),m_imagedisplay,SLOT(refresh()));

    // setup menu
    QPopupMenu *fileMenu = new QPopupMenu(this);
    fileMenu->insertItem("&Save As...",m_imagedisplay,SLOT(saveAs()));
    fileMenu->insertItem("&Exit",this,SLOT(close()));

    QPopupMenu *viewMenu = new QPopupMenu(this);
    m_bwThumbnails = new PopupMenuBool(viewMenu,"&B&&W Thumbnails",true);
    connect(m_bwThumbnails,SIGNAL(valueChanged(bool)),this,SLOT(reload()));

    viewMenu->insertItem("&Refresh",this,SLOT(refresh()));

    menuBar()->insertItem("&File",fileMenu);
    menuBar()->insertItem("&View",viewMenu);

    // 
    statusBar();

    setCentralWidget(m_splitter);

    m_splitter->setResizeMode(m_iconview,QSplitter::KeepSize);

    // load initial directory
    cd(".",true);
}

MainWindow::~MainWindow() 
{
/*    delete m_listbox;
    delete m_imagedisplay;
    delete m_iconview;
    delete m_splitter;*/
    delete m_bwThumbnails;
}

void
MainWindow::selectionChanged()
{
    QIconViewItem * item = m_iconview->currentItem();
    if( item ) {
	m_imagedisplay->show();
	m_imagedisplay->setImage( cwd()->absFilePath( item->text() ) );
    } else {
	m_imagedisplay->hide();
    }
    updateCaption();
}

void
MainWindow::next()
{
   if( m_history.next() ) {
      m_inHistory = true;
      cd( m_history.current() );
      m_inHistory = false;
   } else {
      QApplication::beep();
   }
}

void
MainWindow::prev()
{
   if( m_history.prev() ) {
      m_inHistory = true;
      cd( m_history.current() );
      m_inHistory = false;
   } else {
      QApplication::beep();
   }
}

// reloads thumbnail images
void
MainWindow::reload()
{
    cd(".",true);
}

bool
MainWindow::refresh()
{
    bool status = true;
    // clean out old thumbnails
    if( cwd()->exists(".tn") ) {
	QDir tnDir(cwd()->filePath(".tn"));
	QPtrListIterator<QFileInfo> fir( *tnDir.entryInfoList() );
	QFileInfo *fi;
	while( (fi = fir.current() ) ) {
	    if( !tnDir.remove(fi->fileName()) ) {
		status = false;
	    }
	    ++fir;
	}
    }
    reload();
    return status;
}

bool
MainWindow::cd(
   const QString& path,
   bool force
   )
{
   QString oldPath = cwd()->absPath();
   if( cwd()->cd(path) ) {
      if( force || cwd()->absPath() != oldPath ) {
	 // clear listbox and show new dir contents
	 files()->clear();
	 m_iconview->clear();
	 m_imagedisplay->hide();
	 const QFileInfoList * fil = cwd()->entryInfoList();
	 QPtrListIterator<QFileInfo> fir(*fil);
	 QFileInfo *fi;
	 int        cnt = 0;
	 QRegExp re( "(.gif|.png|.jpg|.thm|.xpm)$",false );
	 // get number of image files
	 while( (fi = fir.current()) ) {
	    if( -1 != re.search(fi->fileName()) ) {
	       ++cnt;
	    }
	    ++fir;
	 }
	 fir.toFirst();
	 QProgressDialog pd("Generating Thumbnails","Cancel",cnt,
			    this,NULL,true);
	 cnt = 0;
	 while( (fi = fir.current()) ) {
	    if( fi->isDir() ) {
	       if( "." != fi->fileName() ) {
		  files()->insertItem( fi->fileName()+"/" );
	       }
	    } else if( -1 != re.search(fi->fileName()) ) {
	       QIconViewItem *item;
	       if( pd.wasCancelled() ) {
		  item = new QIconViewItem(m_iconview,fi->fileName());
	       } else {
		  item = new QIconViewItem(m_iconview,
					   fi->fileName(),
					   thumbNail(fi->fileName()));
	       }
	       item->setRenameEnabled(true);
	       ++cnt;
	    }
	    pd.setProgress(cnt);
	    qApp->processEvents();
	    ++fir;
	 }
	 statusBar()->message("Ready");
	 emit dirChanged(cwd()->absPath());
	 if( !m_inHistory && cnt ) {
	    m_history.push(cwd()->absPath());
	 }
	 updateCaption();
      }
      return true;
   } else {
      return false;
   }
}

QPixmap
MainWindow::thumbNail(
    const QString& filename
    )
{
   bool saveTn = true; // disable tn saving for now
   QString fullname = ".tn/" + filename;
   if( !cwd()->exists(".tn") && !cwd()->mkdir(".tn") ) {
      saveTn = false;
   } else {
      if( cwd()->exists( fullname ) ) {
	 if( m_bwThumbnails->value() ) {
	    QImage tn( cwd()->filePath(fullname) );
	    bwFilter::convert(tn);
	    return QPixmap( tn );
	 } else {
	    return QPixmap( cwd()->filePath(fullname) );
	 }
      }
   }
   // try to make the thumbNail
   statusBar()->message("Generating thumbnail for: " + filename);
   QImage image(cwd()->filePath(filename));
   QImage tn = image.smoothScale( LCL_thumbWidth,
				  LCL_thumbHeight
				  ,QImage::ScaleMin );
   if( saveTn ) {
      tn.save( cwd()->filePath(fullname),
	       image.imageFormat(cwd()->filePath(filename)) );
   }
   if( m_bwThumbnails->value() ) {
      bwFilter::convert(tn);
   }
   return QPixmap(tn);
}

void
MainWindow::updateCaption()
{
   QString caption = PROGRAM " (" VERSION "): ";
   if( m_imagedisplay->isShown() && m_imagedisplay->filename().length() ) {
      caption += m_imagedisplay->filename();
   } else {
      caption += cwd()->absPath();
   }
   setCaption( caption );
}

PopupMenuBool::PopupMenuBool(
   QPopupMenu    *menu,
   const QString &text,
   bool        initialValue
   )
   : m_value(initialValue), m_menu(menu)
{
   m_index = menu->insertItem(text,this,SLOT(toggle()));
   menu->setCheckable(true);
   connect(menu,SIGNAL(aboutToShow()),this,SLOT(preMenuPost()));
}

void
PopupMenuBool::preMenuPost()
{
   m_menu->setItemChecked(m_index,value());
}

bool
PopupMenuBool::value() const
{
   return m_value;
}

void
PopupMenuBool::toggle()
{
   m_value = !m_value;
   emit valueChanged(value());
}

int main(
    int argc,
    char **argv
    )
{
    QApplication a(argc,argv);
    MainWindow mw(NULL,NULL);
    if( 2 == argc ) {
       mw.cd(argv[1]);
    }
    mw.resize( QSize(500,400) );
    mw.show();
    a.connect(&a,SIGNAL(lastWindowClosed()),&a,SLOT(quit()));
    int ret = 0;
    try {
	ret = a.exec();
    } catch (exception& e) {
	ret = -1;
	cerr << "[error] - unhandled exception in main:\"" << e.what() << "\"" << endl;
    }
    return 0;
}
